#define _CRT_SECURE_NO_WARNINGS

#include <iostream>
#include <algorithm>
#include <vector>
#include <cmath>

#define endl '\n'

using namespace std;

const int INF = (int)1e9;

class PseudoTree {
	vector<int> elem;

public:
	void init(int n) {
		elem.assign(n, -1);
	}
	void update(int x, int val) {
		elem[x] = val;
	}
	int get_prev_geq(int x, int val) {
		for (int i = x; i >= 0; i--)
			if (elem[i] >= val)
				return i;
		return -1;
	}
	int get_past_geq(int x, int val) {
		for (int i = x; i < (int)elem.size(); i++)
			if (elem[i] >= val)
				return i;
		return -1;
	}
	int query(int x) {
		return elem[x];
	}
	int get_min(int l, int r) {
		int ans = INF;
		for (int i = l; i < r; i++)
			ans = min(ans, elem[i]);
		return ans;
	}
};

struct SegmentTree {
	vector<int> vmax, vmin, a;
	int n;

	void recalc(int v, int vl, int vr) {
		if (vr - vl == 1)
			return;
		vmax[v] = max(vmax[2 * v + 1], vmax[2 * v + 2]);
		vmin[v] = min(vmin[2 * v + 1], vmin[2 * v + 2]);
	}
	

public:
	void init(int n) {
		this->n = n;
		vmax.resize(4 * n, -1);
		vmin.assign(4 * n, -1);
		a.assign(n, -1);
	}

	void update(int v, int vl, int vr, int ind, int val) {
		if (ind < vl || vr <= ind)
			return;
		if (vr - vl == 1) {
			vmin[v] = val;
			vmax[v] = val;
			a[ind] = val;
			return;
		}
		int vc = (vl + vr) / 2;
		update(2 * v + 1, vl, vc, ind, val);
		update(2 * v + 2, vc, vr, ind, val);
		recalc(v, vl, vr);
	}

	int get_min(int v, int vl, int vr, int ql, int qr) {
		if (qr <= vl || vr <= ql)
			return INF;
		if (ql <= vl && vr <= qr)
			return vmin[v];
		int vc = (vl + vr) / 2;
		return min(get_min(2 * v + 1, vl, vc, ql, qr), get_min(2 * v + 2, vc, vr, ql, qr));
	}

	int get_first_geq(int v, int vl, int vr, int ql, int qr, int val) {
		if (vmax[v] < val)
			return INF;
		if (qr <= vl || vr <= ql)
			return INF;
		if (vr - vl == 1)
			return vl;
		int vc = (vl + vr) / 2;
		int ans = get_first_geq(2 * v + 1, vl, vc, ql, qr, val);
		if (ans == INF)
			ans = get_first_geq(2 * v + 2, vc, vr, ql, qr, val);
		return ans;
	}

	int get_last_geq(int v, int vl, int vr, int ql, int qr, int val) {
		if (vmax[v] < val)
			return -INF;
		if (qr <= vl || vr <= ql)
			return -INF;
		if (vr - vl == 1)
			return vl;
		int vc = (vl + vr) / 2;
		int ans = get_last_geq(2 * v + 2, vc, vr, ql, qr, val);
		if (ans == -INF)
			ans = get_last_geq(2 * v + 1, vl, vc, ql, qr, val);
		return ans;
	}

	int get_val(int ind) {
		return a[ind];
	}
};

int n, m, q, type, i1, jj1, i2, jj2, k, br_row1, br_col1, br_row2, br_col2, dist1, dist2;
SegmentTree cols, rows;
bool cl_col1, cl_col2, cl_row1, cl_row2;

int main() {
	//freopen("input.txt", "r", stdin);
	//freopen("output.txt", "w", stdout);

	ios_base::sync_with_stdio(0);
	cin.tie(0);
	cout.tie(0);

	cin >> n >> m >> q;
	cols.init(m);
	rows.init(n);

	for (int t = 0; t < q; t++) {
		cin >> type;
		if (type == 1) {
			cin >> i1;
			i1--;
			rows.update(0, 0, n, i1, t);
			continue;
		}
		if (type == 2) {
			cin >> jj1;
			jj1--;
			cols.update(0, 0, m, jj1, t);
			continue;
		}

		if (type == 3) {
			cin >> i1 >> jj1 >> i2 >> jj2 >> k;
			i1--;
			jj1--;
			i2--;
			jj2--;

			cl_row1 = t - rows.get_val(i1) <= k;
			cl_col1 = t - cols.get_val(jj1) <= k;
			cl_row2 = t - rows.get_val(i2) <= k;
			cl_col2 = t - cols.get_val(jj2) <= k;
			if (cl_row1 && i1 == i2) {
				cout << abs(jj1 - jj2) << endl;
				continue;
			}
			if (cl_col1 && jj1 == jj2) {
				cout << abs(i1 - i2) << endl;
				continue;
			}
			if (cl_row1 && cl_row2 && abs(i1 - i2) == 1) {
				cout << abs(jj1 - jj2) + 1 << endl;
				continue;
			}
			if (cl_col1 && cl_col2 && abs(jj1 - jj2) == 1) {
				cout << abs(i1 - i2) + 1 << endl;
				continue;
			}
			if ((cl_col1 && cl_row2) || (cl_col2 && cl_row1)) {
				cout << abs(i1 - i2) + abs(jj1 - jj2) << endl;
				continue;
			}
			if (rows.get_min(0, 0, n, min(i1, i2), max(i1, i2) + 1) >= t - k || cols.get_min(0, 0, m, min(jj1, jj2), max(jj1, jj2) + 1) >= t - k) {
				cout << abs(i1 - i2) + abs(jj1 - jj2) << endl;
				continue;
			}

			if (cl_row1 && cl_row2) {
				br_col1 = cols.get_last_geq(0, 0, m, 0, max(jj1, jj2) + 1, t - k);
				br_col2 = cols.get_first_geq(0, 0, m, min(jj1, jj2), m, t - k);
				if (br_col1 == -INF)
					dist1 = INF;
				else
					dist1 = abs(i1 - i2) + abs(jj1 - br_col1) + abs(jj2 - br_col1);
				if (br_col2 == INF)
					dist2 = INF;
				else
					dist2 = abs(i1 - i2) + abs(jj1 - br_col2) + abs(jj2 - br_col2);
				if (dist1 != INF || dist2 != INF) {
					cout << min(dist1, dist2) << endl;
					continue;
				}
			}

			if (cl_col1 && cl_col2) {
				br_row1 = rows.get_last_geq(0, 0, n, 0, max(i1, i2) + 1, t - k);
				br_row2 = rows.get_first_geq(0, 0, n, min(i1, i2), n, t - k);
				if (br_row1 == -INF)
					dist1 = INF;
				else
					dist1 = abs(jj1 - jj2) + abs(i1 - br_row1) + abs(i2 - br_row1);
				if (br_row2 == INF)
					dist2 = INF;
				else
					dist2 = abs(jj1 - jj2) + abs(i1 - br_row2) + abs(i2 - br_row2);
				if (dist1 != INF || dist2 != INF) {
					cout << min(dist1, dist2) << endl;
					continue;
				}
			}

			cout << -1 << endl;
		}
	}

	//system("pause");
	return 0;
}