#define PRINT(str) { std::cout << (str) << std::endl; }

#include <iostream>
#include <queue>
#include <algorithm>
#include <climits>

struct Point {
    long row;
    long col;
    long steps;
    long priority;

    bool operator>(Point other) {
        return this->priority > other.priority;
    }
};

int main() {
    long height, width, queries;
    std::cin >> height >> width >> queries;

    long* last_cleaned_row = new long[height];
    std::fill(last_cleaned_row, last_cleaned_row + height, 0);
    long* last_cleaned_col = new long[width];
    std::fill(last_cleaned_col, last_cleaned_col + width, 0);

    bool* row_available = new bool[height];
    bool* col_available = new bool[width];
    for (long query = 0; query < queries; query++) {
        long type;
        std::cin >> type;

        switch (type) {
            case 1:
                long row;
                std::cin >> row;

                last_cleaned_row[row - 1] = query + 1;
                break;
            case 2:
                long col;
                std::cin >> col;

                last_cleaned_col[col - 1] = query + 1;
                break;
            case 3:
#define SNOW_AT(r, c) (query + 1 - std::max(last_cleaned_row[r], last_cleaned_col[c]))

                long row1, col1, row2, col2, a;
                std::cin >> row1 >> col1 >> row2 >> col2 >> a;

                if (SNOW_AT(row1 - 1, col1 - 1) > a) {
                    std::cout << "-1" << std::endl;
                    continue;
                }

                long min_row = std::min(row1, row2) - 1;
                long max_row = std::max(row1, row2) - 1;
                long min_col = std::min(col1, col2) - 1;
                long max_col = std::max(col1, col2) - 1;

                for (long row = 0; row < height; row++)
                    row_available[row] = last_cleaned_row[row] >= query + 1 - a;
                for (long col = 0; col < width; col++)
                    col_available[col] = last_cleaned_col[col] >= query + 1 - a;

                long path = LONG_MAX;

                long path1 = LONG_MAX;
                for (long row = min_row; row <= max_row; row++) {
                    if (row_available[row] && (col_available[col1 - 1] || row1 - 1 == row) && (col_available[col2 - 1] || row2 - 1 == row)) {
                        path1 = std::abs(row1 - 1 - row) + std::abs(col1 - col2) + std::abs(row2 - 1 - row);
                        break;
                    }
                }
                if (path1 == LONG_MAX) {
                    for (long row = min_row - 1; row >= 0; row--) {
                        if (row_available[row] && (col_available[col1 - 1] || row1 - 1 == row) && (col_available[col2 - 1] || row2 - 1 == row)) {
                            path1 = std::abs(row1 - 1 - row) + std::abs(col1 - col2) + std::abs(row2 - 1 - row);
                            break;
                        }
                    }
                    if (path1 == LONG_MAX) {
                        for (long row = max_row + 1; row < height; row++) {
                            if (row_available[row] && (col_available[col1 - 1] || row1 - 1 == row) && (col_available[col2 - 1] || row2 - 1 == row)) {
                                path1 = std::abs(row1 - 1 - row) + std::abs(col1 - col2) + std::abs(row2 - 1 - row);
                                break;
                            }
                        }
                    }
                }

                path = std::min(path, path1);

                long path2 = LONG_MAX;
                for (long col = min_col; col <= max_col; col++) {
                    if (col_available[col] && (row_available[row1 - 1] || col1 - 1 == col) && (row_available[row2 - 1] || col2 - 1 == col)) {
                        path2 = std::abs(col1 - 1 - col) + std::abs(row1 - row2) + std::abs(col2 - 1 - col);
                        break;
                    }
                }
                if (path2 == LONG_MAX) {
                    for (long col = min_col - 1; col >= 0; col--) {
                        if (col_available[col] && (row_available[row1 - 1] || col1 - 1 == col) && (row_available[row2 - 1] || col2 - 1 == col)) {
                            path2 = std::abs(col1 - 1 - col) + std::abs(row1 - row2) + std::abs(col2 - 1 - col);
                            break;
                        }
                    }
                    if (path2 == LONG_MAX) {
                        for (long col = max_col + 1; col < width; col++) {
                            if (col_available[col] && (row_available[row1 - 1] || col1 - 1 == col) && (row_available[row2 - 1] || col2 - 1 == col)) {
                                path2 = std::abs(col1 - 1 - col) + std::abs(row1 - row2) + std::abs(col2 - 1 - col);
                                break;
                            }
                        }
                    }
                }

                path = std::min(path, path2);

                for (long row = min_row; row <= max_row; row++) {
                    if (!row_available[row])
                        break;
                    if (row == max_row) {
                        path = std::min(path, std::abs(col1 - col2) + std::abs(row1 - row2));
                        break;
                    }
                }

                for (long col = min_col; col <= max_col; col++) {
                    if (!col_available[col])
                        break;
                    if (col == max_col) {
                        path = std::min(path, std::abs(col1 - col2) + std::abs(row1 - row2));
                        break;
                    }
                }

                if (path == LONG_MAX) {
                    std::cout << "-1" << std::endl;
                } else {
                    std::cout << path << std::endl;
                }

//                std::priority_queue<Point, std::vector<Point>, std::greater<>> open = {};
//                // std::queue<Point> open;
//                // std::fill(close, close + height * width, false);
//                PUSH_POINT(row1 - 1, col1 - 1, 0);
//
//                long result = -1;
//                while (!open.empty()) {
//                    Point next = open.top();
//                    open.pop();
//
//                    // std::cout << "Next point is " << next.row << ", " << next.col << std::endl;
//
//                    if (next.row == row2 - 1 && next.col == col2 - 1) {
//                        result = next.steps;
//                        break;
//                    }
//
//                    if (next.row > 0 && !close[(next.row - 1) * width + next.col] && SNOW_AT(next.row - 1, next.col) <= a) {
//                        close[(next.row - 1) * width + next.col] = true;
//                        PUSH_POINT(next.row - 1, next.col, next.steps + 1);
//                    }
//                    if (next.row < height - 1 && !close[(next.row + 1) * width + next.col] && SNOW_AT(next.row + 1, next.col) <= a) {
//                        close[(next.row + 1) * width + next.col] = true;
//                        PUSH_POINT(next.row + 1, next.col, next.steps + 1);
//                    }
//                    if (next.col > 0 && !close[next.row * width + next.col - 1] && SNOW_AT(next.row, next.col - 1) <= a) {
//                        close[next.row * width + next.col - 1] = true;
//                        PUSH_POINT(next.row, next.col - 1, next.steps + 1);
//                    }
//                    if (next.col < width - 1 && !close[next.row * width + next.col + 1] && SNOW_AT(next.row, next.col + 1) <= a) {
//                        close[next.row * width + next.col + 1] = true;
//                        PUSH_POINT(next.row, next.col + 1, next.steps + 1);
//                    }
//                }
//
//                std::cout << result << std::endl;

                break;
        }
    }

    return 0;
}