#include <iostream>
#include <vector>
#include <algorithm>

using namespace std;

const int INF = 1e9;
const int MAX = 1e6 + 2;

int tr_max[MAX * 4];
int tc_max[MAX * 4];
int tr_min[MAX * 4];
int tc_min[MAX * 4];

void set_val_max(int v, int l, int r, int ind, int val, int *t) {
    if (r - l == 1) {
        t[v] = val;
        return;
    }
    int m = (l + r) >> 1;
    if (ind < m) {
        set_val_max(v << 1, l, m, ind, val, t);
    } else {
        set_val_max((v << 1) + 1, m, r, ind, val, t);
    }
    t[v] = max(t[v << 1], t[(v << 1) + 1]);
}

void set_val_min(int v, int l, int r, int ind, int val, int *t) {
    if (r - l == 1) {
        t[v] = val;
        return;
    }
    int m = (l + r) >> 1;
    if (ind < m) {
        set_val_min(v << 1, l, m, ind, val, t);
    } else {
        set_val_min((v << 1) + 1, m, r, ind, val, t);
    }
    t[v] = min(t[v << 1], t[(v << 1) + 1]);
}

int get_min(int v, int l, int r, int ql, int qr, int *t) {
    if (ql >= r || qr <= l) {
        return INF;
    }
    if (ql <= l && qr >= r) {
        return t[v];
    }
    int m = (l + r) >> 1;
    auto left = get_min(v << 1, l, m, ql, qr, t);
    auto right = get_min((v << 1) + 1, m, r, ql, qr, t);
    return min(left, right);
}

int get_max(int v, int l, int r, int ql, int qr, int *t) {
    if (ql >= r || qr <= l) {
        return 0;
    }
    if (ql <= l && qr >= r) {
        return t[v];
    }
    int m = (l + r) >> 1;
    auto left = get_max(v << 1, l, m, ql, qr, t);
    auto right = get_max((v << 1) + 1, m, r, ql, qr, t);
    return max(left, right);
}

int fast_row(int r1, int r2, int* tr, int t, int n) {
    if (r1 > r2) {
        swap(r1, r2);
    }
    int base = abs(r1 - r2);
    if (get_max(1, 0, n + 1, r1, r2 + 1, tr) >= t) {
        return base;
    }
    int up = INF, down = INF;
    int l = 0, r = r1;
    while (r - l > 1) {
        int x = (l + r) >> 1;
        int val = get_max(1, 0, n + 1, x, r1, tr);
        if (val >= t) {
            l = x;
        } else {
            r = x;
        }
    }
    if (l)
        up = r1 - l;
    l = r2;
    r = n + 1;
    while (r - l > 1) {
        int x = (l + r) >> 1;
        int val = get_max(1, 0, n + 1, l, x + 1, tr);
        if (val >= t) {
            r = x;
        } else {
            l = x;
        }
    }
    if (r < n + 1)
        down = r - r2;
    if (up == INF && down == INF) {
        return INF;
    }
    int ans = base + 2 * min(up, down);
    return ans;
}

int main() {
    ios::sync_with_stdio(false);
    //for (int test = 0; test <= 1000; test++) {
    int n, m, q = 1000;
    //n = rand() % 10 + 2;
    //m = rand() % 10 + 2;
    cin >> n >> m >> q;
    vector<int> r(n + 1);
    vector<int> c(m + 1);
    for (int i = 1; i <= q; i++) {
        int t = rand() % 3 + 1;
        cin >> t;
        if (t == 1) {
            int x = rand() % n + 1;
            cin >> x;
            r[x] = i;
            set_val_max(1, 0, n + 1, x, i, tr_max);
            set_val_min(1, 0, n + 1, x, i, tr_min);
        } else if (t == 2) {
            int x = rand() % m + 1;
            cin >> x;
            c[x] = i;
            set_val_max(1, 0, m + 1, x, i, tc_max);
            set_val_min(1, 0, m + 1, x, i, tc_min);
        } else {
            int r1, c1, r2, c2, k;
            /*r1 = rand() % n + 1;
            c1 = rand() % m + 1;
            r2 = rand() % n + 1;
            c2 = rand() % m + 1;
            k = rand() % q;*/
            cin >> r1 >> c1 >> r2 >> c2 >> k;
            int t = i - k;
            if ((r[r1] >= t && c[c2] >= t) || (r[r2] >= t && c[c1] >= t)) {
                int ans = abs(r1 - r2) + abs(c1 - c2);
                cout << ans << '\n';
                continue;
            }
            if (get_min(1, 0, m + 1, min(c1, c2), max(c1, c2) + 1, tc_min) >= t) {
                int ans = abs(r1 - r2) + abs(c1 - c2);
                cout << ans << '\n';
                continue;
            }
            if (get_min(1, 0, n + 1, min(r1, r2), max(r1, r2) + 1, tr_min) >= t) {
                int ans = abs(r1 - r2) + abs(c1 - c2);
                cout << ans << '\n';
                continue;
            }
            int ans = INF;
            if (c[c1] >= t && c[c2] >= t) {
                ans = min(ans, fast_row(r1, r2, tr_max, t, n) + abs(c1 - c2));
            }
            if (r[r1] >= t && r[r2] >= t) {
                ans = min(ans, fast_row(c1, c2, tc_max, t, m) + abs(r1 - r2));
            }
            cout << (ans == INF ? -1 : ans) << '\n';
        }
    //}
    }
}
/*
4 3 9
3 2 1 3 3 1
3 2 1 3 3 1
2 1
2 3
1 1
3 2 1 3 3 3
3 2 1 3 3 3
2 2
3 2 1 3 3 6
*/
