#include <bits/stdc++.h>

using namespace std;

const int INF = 1e9;

#pragma GCC optimize("Ofast")
#pragma GCC target("sse,sse2,sse3,ssse3,sse4,popcnt,avx,avx2,tune=native")
#pragma GCC optimize("unroll-loops")
#pragma GCC optimize("fast-math")

struct SegmentsTree {
    int size;
    vector<int> tmax, tmin;

    SegmentsTree(int n) {
        size = 1 << ((int) ceil(log2(n)));
        tmax.resize(2 * size);
        tmin.resize(2 * size);
        for (int i = 0; i < n; i++) {
            update(i, 0);
        }
    }

    int get_max(int v, int l, int r, int A, int B) {
        if (r < A || l > B) {
            return 0;
        }
        if (A <= l && r <= B) {
            return tmax[v];
        }
        int m = (l + r) / 2;
        return max(get_max(2 * v, l, m, A, B), get_max(2 * v + 1, m + 1, r, A, B));
    }

    int get_min(int v, int l, int r, int A, int B) {
        if (r < A || l > B) {
            return INF;
        }
        if (A <= l && r <= B) {
            return tmin[v];
        }
        int m = (l + r) / 2;
        return min(get_min(2 * v, l, m, A, B), get_min(2 * v + 1, m + 1, r, A, B));
    }

    int get(int l, int r, int t) {
        if (l > r) {
            swap(l, r);
        }
        return t == 0 ? get_max(1, 0, size - 1, l, r) : get_min(1, 0, size - 1, l, r);
    }

    int gdl(int v, int l, int r, int L, int R, int x) {
        if (l == r) {
            return l <= R ? l : L - 1;
        }
        int m = (l + r) / 2;
        int t1 = get_max(2 * v, l, m, L, R);
        int t2 = get_max(2 * v + 1, m + 1, r, L, R);
        if (t2 >= x) {
            return gdl(2 * v + 1, m + 1, r, L, R, x);
        } else if (t1 >= x) {
            return gdl(2 * v, l, m, L, R, x);
        } else {
            return L - 1;
        }
    }

    int gdl(int L, int R, int x) {
        return gdl(1, 0, size - 1, L, R, x);
    }

    int gdr(int v, int l, int r, int L, int R, int x) {
        if (l == r) {
            return l >= L ? l : R + 1;
        }
        int m = (l + r) / 2;
        int t1 = get_max(2 * v, l, m, L, R);
        int t2 = get_max(2 * v + 1, m + 1, r, L, R);
        if (t1 >= x) {
            return gdr(2 * v, l, m, L, R, x);
        } else if (t2 >= x) {
            return gdr(2 * v + 1, m + 1, r, L, R, x);
        } else {
            return R + 1;
        }

    }

    int gdr(int L, int R, int x) {
        return gdr(1, 0, size - 1, L, R, x);
    }

    void update(int v, int l, int r, int i, int x) {
        if (i < l || r < i) {
            return;
        }
        if (l == r) {
            tmax[v] = x;
            tmin[v] = x;
            return;
        }
        int m = (l + r) / 2;
        update(2 * v, l, m, i, x);
        update(2 * v + 1, m + 1, r, i, x);
        tmax[v] = max(tmax[2 * v], tmax[2 * v + 1]);
        tmin[v] = min(tmin[2 * v], tmin[2 * v + 1]);
    }

    void update(int i, int x) {
        update(1, 0, size - 1, i, x);
    }
};

int32_t main() {
#ifdef _DEBUG
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#endif
    int n, m, q;
    cin >> n >> m >> q;
    auto treex = SegmentsTree(n);
    auto treey = SegmentsTree(m);
    int t, r, c, r1, c1, r2, c2, k;
    for (int T = 1; T <= q; T++) {
        cin >> t;
        if (t == 1) {
            cin >> r;
            treex.update(r - 1, T);
        } else if (t == 2) {
            cin >> c;
            treey.update(c - 1, T);
        } else {
            cin >> r1 >> c1 >> r2 >> c2 >> k;
            r1--;
            c1--;
            r2--;
            c2--;
            bool ok1r = T - treex.get(r1, r1, 0) <= k,
                    ok1c = T - treey.get(c1, c1, 0) <= k,
                    ok2r = T - treex.get(r2, r2, 0) <= k,
                    ok2c = T - treey.get(c2, c2, 0) <= k;
            int ans = INF;
            if ((ok1r && ok2c) || (ok1c && ok2r)) {
                ans = abs(r2 - r1) + abs(c2 - c1);
            } else if (ok1r && ok2r) {
                if (T - treex.get(r1, r2, 1) <= k || T - treey.get(c1, c2, 0) <= k) {
                    ans = abs(r2 - r1) + abs(c2 - c1);
                } else {
                    // TODO: replace with tree function

                    int DC = treey.gdl(0, min(c1, c2) - 1, T - k);
                    int UC = treey.gdr(max(c1, c2) + 1, m - 1, T - k);
                    if (DC != -1 && UC != m) {
                        ans = abs(r1 - r2) + abs(c1 - c2) + min(
                                2 * (min(c1, c2) - DC),
                                2 * (UC - max(c1, c2))
                        );
                    } else if (DC != -1) {
                        ans = abs(r1 - r2) + abs(c1 - c2) + 2 * (min(c1, c2) - DC);
                    } else if (UC != m) {
                        ans = abs(r1 - r2) + abs(c1 - c2) + 2 * (UC - max(c1, c2));
                    }
                }
            } else if (ok1c && ok2c) {
                if (T - treey.get(c1, c2, 1) <= k || T - treex.get(r1, r2, 0) <= k) {
//                    cout << treey.get(c1, c2, 1) << ' ' << treex.get(r1, r2, 0) << endl;
                    ans = abs(r2 - r1) + abs(c2 - c1);
                } else {
                    // TODO: replace with tree function

                    int DR = treex.gdl(0, min(r1, r2) - 1, T - k);
                    int UR = treex.gdr(max(r1, r2) + 1, n - 1, T - k);
//                    cout << DR << ' ' << UR << endl;
                    if (DR != -1 && UR != n) {
                        ans = abs(r1 - r2) + abs(c1 - c2) + min(
                                2 * (min(r1, r2) - DR),
                                2 * (UR - max(r1, r2))
                        );
                    } else if (DR != -1) {
                        ans = abs(r1 - r2) + abs(c1 - c2) + 2 * (min(r1, r2) - DR);
                    } else if (UR != n) {
                        ans = abs(r1 - r2) + abs(c1 - c2) + 2 * (UR - max(r1, r2));
                    }
                }
            }
            if (ans != INF) {
                cout << ans;
            } else {
                cout << -1;
            }
            cout << '\n';
        }
    }
    return 0;
}