#include <iostream>
#include <vector>
#include <algorithm>

using namespace std;

#define endl '\n'

struct node {
	int val;
	int mx;
	int l, r;
	node* left;
	node* right;
	node(int val, int l, int r, node* left = nullptr, node* right = nullptr) :
		val(val), mx(val), l(l), r(r), left(left), right(right) {}
};

node* build(int l, int r) {
	if (r - l == 1) {
		return new node(0, l, r);
	}
	int m = (l + r) / 2;
	auto v = new node(0, l, r, build(l, m), build(m, r));
	return v;
}

const int INF = 1e6;

int get_min(node* v, int l, int r) {
	if (v->r <= l || v->l >= r) return INF;
	if (v->l >= l && v->r <= r) {
		return v->val;
	}
	return min(get_min(v->left, l, r), get_min(v->right, l, r));
}

int get_max(node* v, int l, int r) {
	if (v->r <= l || v->l >= r) return 0;
	if (v->l >= l && v->r <= r) {
		return v->mx;
	}
	return max(get_max(v->left, l, r), get_max(v->right, l, r));
}

int get_max1(node* v, int l, int r, int x) {
	if (v->r <= l || v->l >= r) return INF;
	if (v->l >= l && v->r <= r) {
		if (v->r - v->l == 1) {
			if (v->mx >= x) {
				return v->l;
			}
			return INF;
		}
		
		if (v->right->mx >= x) {
			return get_max1(v->right, l, r, x);
		}
		return get_max1(v->left, l, r, x);
	}

	int res = get_max1(v->right, l, r, x);
	if (res != INF) {
		return res;
	}
	return get_max1(v->left, l, r, x);
}

int get_max2(node* v, int l, int r, int x) {
	if (v->r <= l || v->l >= r) return INF;
	if (v->l >= l && v->r <= r) {
		if (v->r - v->l == 1) {
			if (v->mx >= x) {
				return v->r - 1;
			}
			return INF;
		}

		if (v->left->mx >= x) {
			return get_max2(v->left, l, r, x);
		}
		return get_max2(v->right, l, r, x);
	}

	int res = get_max2(v->left, l, r, x);
	if (res != INF) {
		return res;
	}
	return get_max2(v->right, l, r, x);
}

void relax(node* v) {
	v->val = min(v->left->val, v->right->val);
	v->mx = max(v->left->mx, v->right->mx);
	return;
}

void change(node* v, int l, int r, int x) {
	if (v->r <= l || v->l >= r) return;
	if (v->l >= l && v->r <= r) {
		v->val = x;
		v->mx = x;
		return;
	}
	change(v->left, l, r, x);
	change(v->right, l, r, x);
	relax(v);
	return;
}

int s(int x1, int x2, int x) {
	if (x >= min(x1, x2) && x <= max(x1, x2)) return 0;
	return min(abs(x1 - x), abs(x2 - x));
}

int main() {
	ios_base::sync_with_stdio(false);
	cin.tie(0);

	int n, m, q;
	cin >> n >> m >> q;

	vector<int> str(n);
	vector<int> stb(m);
	node* rroot = build(0, n);
	node* broot = build(0, m);
	
	int t = 1;
	for (int i = 0; i < q; i++, t++) {
		int type;
		cin >> type;
		if (type == 1) {
			int ind;
			cin >> ind;
			str[--ind] = t;
			change(rroot, ind, ind + 1, t);
		}
		else if (type == 2) {
			int ind;
			cin >> ind;
			stb[--ind] = t;
			change(broot, ind, ind + 1, t);
		}
		else {
			int x1, y1, x2, y2, k;
			cin >> x1 >> y1 >> x2 >> y2 >> k;

			--x1; --y1; --x2; --y2;
			if ((str[x1] < t - k && stb[y1] < t - k) ||
				(str[x2] < t - k && stb[y2] < t - k)) {
				cout << -1 << endl;
				continue;
			}

			if ((str[x1] >= t - k && stb[y2] >= t - k) ||
				(stb[y1] >= t - k && str[x2] >= t - k)) {
				cout << abs(x1 - x2) + abs(y1 - y2) << endl;
				continue;
			}

			if (str[x1] >= t - k && str[x2] >= t - k) {
				if (y1 > y2) {
					swap(x1, x2);
					swap(y1, y2);
				}

				int s = min(y1 - get_max1(broot, 0, y1 + 1, t - k), get_max2(broot, y2, m, t - k) - y2);

				int ans = s == INF ? -1 : 2 * s;
				if (get_max(broot, y1, y2 + 1) >= t - k) {
					ans = 0;
				}

				int x = get_min(rroot, min(x1, x2), max(x1, x2) + 1);

				if (x >= t - k) {
					ans = 0;
				}
				else if (ans == -1) {
					cout << -1 << endl;
					continue;
				}

				cout << abs(x1 - x2) + abs(y1 - y2) + ans << endl;
			}
			else {
				if (x1 > x2) {
					swap(x1, x2);
					swap(y1, y2);
				}

				int s = min(x1 - get_max1(rroot, 0, x1 + 1, t - k), get_max2(rroot, x2, n, t - k) - x2);

				int ans = s < 0 ? -1 : 2 * s;
				if (get_max(rroot, x1, x2 + 1) >= t - k) {
					ans = 0;
				}

				int x = get_min(broot, min(y1, y2), max(y1, y2) + 1);
				/*int x = INF;
				for (int j = min(y1, y2); j <= max(y1, y2); j++) {
					x = min(x, stb[j]);
				}*/

				if (x >= t - k) {
					ans = 0;
				}
				else if (ans == -1) {
					cout << -1 << endl;
					continue;
				}

				cout << abs(x1 - x2) + abs(y1 - y2) + ans << endl;
			}
		}
	}
}