#include <bits/stdc++.h>

using namespace std;

typedef long long ll;

mt19937 gen(2007);

struct node
{
    node* l = nullptr;
    node* r = nullptr;
    int idx, val, maxv, minv, y;
    int cnt = 1;

    node(int i, int v): idx(i), val(v), maxv(v), minv(v), y(gen()) {}
};

typedef node* tree;

int cnt(tree t)
{
    if (t)
        return t->cnt;
    else
        return 0;
}

int maxv(tree t)
{
    if (t)
        return t->maxv;
    else
        return -1e9;
}

int minv(tree t)
{
    if (t)
        return t->minv;
    else
        return 1e9;
}

void relax(tree t)
{
    if (!t)
        return;

    t->cnt = cnt(t->l) + cnt(t->r) + 1;
    t->maxv = max(t->val, max(maxv(t->l), maxv(t->r)));
    t->minv = min(t->val, min(minv(t->l), minv(t->r)));
}

pair<tree, tree> split_kth(tree t, int k) // k, n - k
{
    if (!t)
        return {nullptr, nullptr};

    if (cnt(t->l) >= k)
    {
        auto p = split_kth(t->l, k);
        t->l = p.second;
        relax(t);
        return {p.first, t};
    }
    else
    {
        auto p = split_kth(t->r, k - cnt(t->l) - 1);
        t->r = p.first;
        relax(t);
        return {t, p.second};
    }
}

tree merge(tree l, tree r)
{
    if (!l)
        return r;
    if (!r)
        return l;

    if (l->y > r->y)
    {
        l->r = merge(l->r, r);
        relax(l);
        return l;
    }
    else
    {
        r->l = merge(l, r->l);
        relax(r);
        return r;
    }
}

void update(tree t, int x, int v)
{
    if (t->idx == x)
        t->val = v;
    else if (t->idx < x)
        update(t->r, x, v);
    else
        update(t->l, x, v);

    relax(t);
}

int find_last_bigger(tree t, int x)
{
    if (!t || maxv(t) < x)
        return -1;
    else if (maxv(t->r) >= x)
        return find_last_bigger(t->r, x);
    else if (t->val >= x)
        return t->idx;
    else
        return find_last_bigger(t->l, x);
}

int find_first_bigger(tree t, int x)
{
    if (!t || maxv(t) < x)
        return -1;
    else if (maxv(t->l) >= x)
        return find_first_bigger(t->l, x);
    else if (t->val >= x)
        return t->idx;
    else
        return find_first_bigger(t->r, x);
}

int at(tree t, int x)
{
    if (t->idx == x)
        return t->val;
    else if (t->idx < x)
        return at(t->r, x);
    else
        return at(t->l, x);
}

int main()
{
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);

    int n, m, q;
    cin >> n >> m >> q;

    tree rows = nullptr;
    tree cols = nullptr;
    for (int i = 0; i < n; i++)
        rows = merge(rows, new node(i, -1));
    for (int i = 0; i < m; i++)
        cols = merge(cols, new node(i, -1));

    for (int i = 0; i < q; i++)
    {
        int t;
        cin >> t;

        if (t == 1)
        {
            int x;
            cin >> x;
            update(rows, x - 1, i);
        }
        else if (t == 2)
        {
            int x;
            cin >> x;
            update(cols, x - 1, i);
        }
        else
        {
            int x1, y1, x2, y2, k;
            cin >> x1 >> y1 >> x2 >> y2 >> k;
            x1--;
            y1--;
            x2--;
            y2--;

            auto rp = split_kth(rows, max(x1, x2) + 1);
            auto rp1 = split_kth(rp.first, min(x1, x2)); // rp1.first, RP1.SECOND,
                                                         // rp.second
            auto cp = split_kth(cols, max(y1, y2) + 1);
            auto cp1 = split_kth(cp.first, min(y1, y2)); // cp1.first, CP1.SECOND,
                                                         // cp.second

            if ((minv(rp1.second) >= i - k || minv(cp1.second) >= i - k) ||
                (at(rp1.second, x1) >= i - k && at(cp1.second, y2) >= i - k) ||
                (at(rp1.second, x2) >= i - k && at(cp1.second, y1) >= i - k))
            {
                cout << abs(x2 - x1) + abs(y2 - y1) << endl;
                rp.first = merge(rp1.first, rp1.second);
                cp.first = merge(cp1.first, cp1.second);
                rows = merge(rp.first, rp.second);
                cols = merge(cp.first, cp.second);
                continue;
            }

            rp.first = merge(rp1.first, rp1.second);
            cp.first = merge(cp1.first, cp1.second);

            int mindist = 1e9;

            if (at(cp.first, y1) >= i - k && at(cp.first, y2) >= i - k)
            {
                int z = find_last_bigger(rp.first, i - k);
                if (z != -1)
                    mindist = min(mindist, abs(x1 - z) + abs(x2 - z) + abs(y1 - y2));
                z = find_first_bigger(rp.second, i - k);
                if (z != -1)
                    mindist = min(mindist, abs(x1 - z) + abs(x2 - z) + abs(y1 - y2));
            }

            if (at(rp.first, x1) >= i - k && at(rp.first, x2) >= i - k)
            {
                int z = find_last_bigger(cp.first, i - k);
                if (z != -1)
                    mindist = min(mindist, abs(y1 - z) + abs(y2 - z) + abs(x1 - x2));
                z = find_first_bigger(cp.second, i - k);
                if (z != -1)
                    mindist = min(mindist, abs(y1 - z) + abs(y2 - z) + abs(x1 - x2));
            }

            if (mindist == 1e9)
                cout << "-1" << endl;
            else
                cout << mindist << endl;

            rows = merge(rp.first, rp.second);
            cols = merge(cp.first, cp.second);
        }
    }

    return 0;
}