#include <iostream>
#include <vector>

using namespace std;

const int MAX_N = 1 << 20, INF = 1e9 + 7;
int vert[MAX_N], gor[MAX_N];
int n, m;
pair <int, int> t[2][2 * MAX_N];

void relax(int c, int pos) {
	t[c][pos].first = min(t[c][2 * pos].first, t[c][2 * pos + 1].first);
	t[c][pos].second = max(t[c][2 * pos].second, t[c][2 * pos + 1].second);
}

void update(int c, int pos, int v) {
	pos += MAX_N - 1;
	t[c][pos] = {v, v};
	while (pos > 1) {
		pos /= 2;
		relax(c, pos);
	}
}

int get_min(int c, int pos, int l, int r, int a, int b) {
	if (l > b || a > r) return INF;
	if (a <= l && r <= b) return t[c][pos].first;
	int m = (l + r) / 2;
	return min(get_min(c, 2 * pos, l, m, a, b), get_min(c, 2 * pos + 1, m + 1, r, a, b));
}

int get_max(int c, int pos, int l, int r, int a, int b) {
	if (l > b || a > r) return -INF;
	if (a <= l && r <= b) return t[c][pos].second;
	int m = (l + r) / 2;
	return max(get_max(c, 2 * pos, l, m, a, b), get_max(c, 2 * pos + 1, m + 1, r, a, b));
}

int get_point(int x, int y) {
	return max(gor[x], vert[y]);
}

pair <int, int> get_nearest_vert(int y, int k) {
	pair <int, int> ans(-1, -1);
	int left = 0, right = y, mid;
	while (right - left > 1) {
		mid = (left + right) / 2;
		if (get_max(1, 1, 1, MAX_N, mid, y) >= k) left = mid;
		else right = mid;
	}
	if (left != 0) ans.first = left;
	left = y, right = m + 1, mid;
	while (right - left > 1) {
		mid = (left + right) / 2;
		if (get_max(1, 1, 1, MAX_N, y, mid) >= k) right = mid;
		else left = mid;
	}
	if (right != m + 1) ans.second = right;
	return ans;
}

pair <int, int> get_nearest_gor(int y, int k) {
	pair <int, int> ans(-1, -1);
	int left = 0, right = y, mid;
	while (right - left > 1) {
		mid = (left + right) / 2;
		if (get_max(0, 1, 1, MAX_N, mid, y) >= k) left = mid;
		else right = mid;
	}
	if (left != 0) ans.first = left;
	left = y, right = n + 1, mid;
	while (right - left > 1) {
		mid = (left + right) / 2;
		if (get_max(0, 1, 1, MAX_N, y, mid) >= k) right = mid;
		else left = mid;
	}
	if (right != n + 1) ans.second = right;
	return ans;
}

int min_gor(int l, int r) {
	if (l > r) swap(l, r);
	return get_min(0, 1, 1, MAX_N, l, r);
}

int min_vert(int l, int r) {
	if (l > r) swap(l, r);
	return get_min(1, 1, 1, MAX_N, l, r);
}

int main() {
	//freopen("Desktop/input.txt", "r", stdin);
	ios_base :: sync_with_stdio(0);
	cin.tie(0);
	int q;
	cin >> n >> m >> q;
	for (int i = 1; i <= n; i++) gor[i] = 0;
	for (int i = 1; i <= m; i++) vert[i] = 0;
	for (int i = 1; i < 2 * MAX_N; i++) t[0][i] = t[1][i] = {0, 0};
	for (int i = 1; i <= q; i++) {
		int t;
		cin >> t;
		if (t == 1) {
			int pos;
			cin >> pos;
			gor[pos] = i;
			update(0, pos, i);
			continue;
		}
		if (t == 2) {
			int pos;
			cin >> pos;
			vert[pos] = i;
			update(1, pos, i);
			continue;
		}
		//cout << 86 << endl;
		int x1, y1, x2, y2, k;
		cin >> x1 >> y1 >> x2 >> y2 >> k;
		k = i - k;
		if (get_point(x1, y1) < k || get_point(x2, y2) < k) {
			cout << -1 << "\n";
			continue;
		}
		if (gor[x1] >= k && vert[y2] >= k) {
			cout << abs(x1 - x2) + abs(y1 - y2) << "\n";
			continue;
		}
		if (gor[x2] >= k && vert[y1] >= k) {
			cout << abs(x1 - x2) + abs(y1 - y2) << "\n";
			continue;
		}
		//cout << 101 << endl;
		int ans = MAX_N * 4;
		if (gor[x1] >= k && gor[x2] >= k) {
			pair <int, int> a = get_nearest_vert(y1, k);
			pair <int, int> b = get_nearest_vert(y2, k);
			vector <int> now = {a.first, a.second, b.first, b.second};
			for (int elem : now) {
				if (elem == -1) continue;
				ans = min(ans, abs(y1 - elem) + abs(elem - y2) + abs(x1 - x2));
			}
			int t = min_gor(x1, x2);
			if (t >= k) ans = min(ans, abs(x1 - x2) + abs(y1 - y2));
		}
		if (vert[y1] >= k && vert[y2] >= k) {
			pair <int, int> a = get_nearest_gor(x1, k);
			pair <int, int> b = get_nearest_gor(x2, k);
			vector <int> now = {a.first, a.second, b.first, b.second};
			for (int elem : now) {
				if (elem == -1) continue;
				ans = min(ans, abs(x1 - elem) + abs(elem - x2) + abs(y1 - y2));
			}
			int t = min_vert(y1, y2);
			if (t >= k) ans = min(ans, abs(x1 - x2) + abs(y1 - y2));
		}
		if (ans == MAX_N * 4) cout << -1 << "\n";
		else cout << ans << "\n";
	}
}
