#include <iostream>
#include <vector>
#include <algorithm>
#include <queue>

using namespace std;

const int inf = 1e9;

struct cor {
	int x, y;
	cor() {}
	cor(int x, int y) : x(x), y(y) {}
	void input() {
		cin >> x >> y;
		--x;
		--y;
	}
};

struct seg_tree {
	vector<int> arr, left_ind, right_ind;
	void build(int v, int left, int right) {
		left_ind[v] = left;
		right_ind[v] = right - 1;
		if (left + 1 != right) {
			int mid = (left + right) / 2;
			build(2 * v, left, mid);
			build(2 * v + 1, mid, right);
		}
	}
	seg_tree(int n) : arr(4 * n), left_ind(4 * n), right_ind(4 * n) {
		build(1, 0, n);
	}
	void update(int v, int left, int right, int pos, int val) {
		if (left + 1 == right)
			arr[v] = val;
		else {
			int mid = (left + right) / 2;
			if (pos < mid)
				update(2 * v, left, mid, pos, val);
			else
				update(2 * v + 1, mid, right, pos, val);
			if (arr[left_ind[2 * v]] <= arr[left_ind[2 * v + 1]])
				left_ind[v] = left_ind[2 * v];
			else
				left_ind[v] = left_ind[2 * v + 1];
			if (arr[right_ind[2 * v]] < arr[right_ind[2 * v + 1]])
				right_ind[v] = right_ind[2 * v];
			else
				right_ind[v] = right_ind[2 * v + 1];
		}
	}
	int get_left_min(int v, int left, int right, int qleft, int qright) {
		if ((right <= qleft) || (qright <= left))
			return -1;
		if ((qleft <= left) && (right <= qright))
			return left_ind[v];
		int mid = (left + right) / 2;
		int ans_left = get_left_min(2 * v, left, mid, qleft, qright);
		int ans_right = get_left_min(2 * v + 1, mid, right, qleft, qright);
		if (ans_left == -1)
			return ans_right;
		else if (ans_right == -1)
			return ans_left;
	}
	int get_right_min(int v, int left, int right, int qleft, int qright) {
		if ((right <= qleft) || (qright <= left))
			return -1;
		if ((qleft <= left) && (right <= qright))
			return right_ind[v];
		int mid = (left + right) / 2;
		int ans_left = get_right_min(2 * v, left, mid, qleft, qright);
		int ans_right = get_right_min(2 * v + 1, mid, right, qleft, qright);
		if (ans_left == -1)
			return ans_right;
		else if (ans_right == -1)
			return ans_left;
	}
};

int main() {
	int n, m, q;
	cin >> n >> m >> q;
	seg_tree x_line(n), y_line(m);
	for (int k = 1; k <= q; ++k) {
		int type;
		cin >> type;
		if (type == 1) {
			int x;
			cin >> x;
			x_line.update(1, 0, n, x - 1, k);
		}
		else if (type == 2) {
			int y;
			cin >> y;
			y_line.update(1, 0, n, y - 1, k);
		}
		else if (type == 3) {
			cor s, f;
			s.input();
			f.input();
			int can;
			cin >> can;
			int ans = inf;
			if ((k - x_line.arr[s.x] <= can) && (k - y_line.arr[f.y] <= can))
				ans = abs(s.x - f.x) + abs(s.y - f.y);
			if ((k - y_line.arr[s.y] <= can) && (k - x_line.arr[f.x] <= can))
				ans = abs(s.x - f.x) + abs(s.y - f.y);
			if ((k - x_line.arr[s.x] <= can) && (k - x_line.arr[f.x] <= can)) {
				int mid_line = y_line.get_left_min(1, 0, n, min(s.y, f.y), max(s.y, f.y) + 1);
				int left_line = y_line.get_right_min(1, 0, n, 0, min(s.y, f.y) + 1);
				int right_line = y_line.get_left_min(1, 0, n, max(s.y, f.y), n);
				if (k - y_line.arr[mid_line] <= can)
					ans = abs(s.x - f.x) + abs(s.y - f.y);
				if (k - y_line.arr[left_line] <= can)
					ans = min(ans, abs(s.y - left_line) + (left_line - f.y) + abs(s.x - f.x));
				if (k - y_line.arr[right_line] <= can)
					ans = min(ans, abs(s.y - right_line) + (right_line - f.y) + abs(s.x - f.x));
			}
			if ((k - y_line.arr[s.y] <= can) && (k - y_line.arr[f.y] <= can)) {
				int mid_line = x_line.get_left_min(1, 0, n, min(s.x, f.x), max(s.x, f.x) + 1);
				int left_line = x_line.get_right_min(1, 0, n, 0, min(s.x, f.x) + 1);
				int right_line = x_line.get_left_min(1, 0, n, max(s.x, f.x), n);
				if (k - x_line.arr[mid_line] <= can)
					ans = abs(s.x - f.x) + abs(s.y - f.y);
				if (k - x_line.arr[left_line] <= can)
					ans = min(ans, abs(s.x - left_line) + (left_line - f.x) + abs(s.y - f.y));
				if (k - y_line.arr[right_line] <= can)
					ans = min(ans, abs(s.x - right_line) + (right_line - f.x) + abs(s.y - f.y));
			}
			if (ans == inf)
				cout << "-1\n";
			else
				cout << ans << "\n";
		}
	}
}
