#include <iostream>
#include <vector>
#include <algorithm>
#include <string>


using namespace std;


const int INF = (int)1e6;


#define left(v) (v << 1) + 1
#define right(v) (v << 1) + 2
#define parent(v) v - 1 >> 1

class MinLST {
    vector<int> data;
    int m;

    void update(int v) {
        if (data[left(v)] < data[right(v)]) {
            data[v] = data[left(v)];
        } else {
            data[v] = data[right(v)];
        }
    }

public:
    MinLST(int n) {
        for (m = 1; m < n; m <<= 1);
        --m;

        data.resize(2 * m + 1);
    }

    void set(int i, int v) {
        i += m;

        data[i] = v;

        while (i) {
            i = parent(i);

            update(i);
        }
    }

    int get(int L, int R) {
        L += m;
        R += m;

        int res = INF;
        while (L < R) {
            if ((L & 1) ^ 1) {
                if (data[L] < res) {
                    res = data[L];
                }

                ++L;
            }

            L = parent(L);

            if (R & 1) {
                if (data[R] < res) {
                    res = data[R];
                }

                --R;
            }

            R = parent(R);
        }

        if (L == R && data[L] < res) {
            res = data[L];
        }

        return res;
    }

    int get(int i) {
        return get(i, i);
    }
};


class MaxLST {
    vector<int> data;
    int m;

    void update(int v) {
        if (data[left(v)] > data[right(v)]) {
            data[v] = data[left(v)];
        } else {
            data[v] = data[right(v)];
        }
    }

public:
    MaxLST(int n) {
        for (m = 1; m < n; m <<= 1);
        --m;

        data.resize(2 * m + 1);
    }

    void set(int i, int v) {
        i += m;

        data[i] = v;

        while (i) {
            i = parent(i);

            update(i);
        }
    }

    int get(int L, int R) {
        L += m;
        R += m;

        int res = 0;
        while (L < R) {
            if ((L & 1) ^ 1) {
                if (data[L] > res) {
                    res = data[L];
                }

                ++L;
            }

            L = parent(L);

            if (R & 1) {
                if (data[R] > res) {
                    res = data[R];
                }

                --R;
            }

            R = parent(R);
        }

        if (L == R && data[L] > res) {
            res = data[L];
        }

        return res;
    }
};


int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(0);

    int n, m, q;
    cin >> n >> m >> q;

    MinLST minc(m), minr(n);
    MaxLST maxc(m), maxr(n);

    int t, r, c, x1, y1, x2, y2, k;
    for (int cq = 1; cq <= q; ++cq) {
        cin >> t;

        if (t == 1) {
            cin >> r;
            --r;

            minr.set(r, cq);
            maxr.set(r, cq);
        } else if (t == 2) {
            cin >> c;
            --c;

            minc.set(c, cq);
            maxc.set(c, cq);
        } else {
            cin >> x1 >> y1 >> x2 >> y2 >> k;
            --x1;
            --y1;
            --x2;
            --y2;

            int tr = cq - k;

            vector<vector<int>> open = {
                {minr.get(x1) >= tr, minc.get(y1) >= tr},
                {minr.get(x2) >= tr, minc.get(y2) >= tr}
            };

            if (!open[0][0] && !open[0][1]) {
                cout << -1 << '\n';

                continue;
            }

            if (!open[1][0] && !open[1][1]) {
                cout << -1 << '\n';

                continue;
            }

            int d = abs(x1 - x2) + abs(y1 - y2);

            if (open[0][0] && open[1][1]) {
                cout << d << '\n';

                continue;
            }

            if (open[1][0] && open[0][1]) {
                cout << d << '\n';

                continue;
            }

            if (open[0][0]) {
                if (x1 > x2) {
                    swap(x1, x2);
                }

                if (minr.get(x1, x2) >= tr) {
                    cout << d << '\n';

                    continue;
                }

                if (y1 > y2) {
                    swap(y1, y2);
                }

                if (maxc.get(0, m - 1) < tr) {
                    cout << -1 << '\n';

                    continue;
                }

                int low = -1, high = max(y1, m - y2 - 1);
                while (high - low > 1) {
                    int mid = (low + high + 1) >> 1;

                    int l = max(0, y1 - mid);
                    int r = min(m - 1, y2 + mid);

                    if (maxc.get(l, r) >= tr) {
                        high = mid;
                    } else {
                        low = mid;
                    }
                }

                cout << d + high * 2 << '\n';
            } else {
                if (y1 > y2) {
                    swap(y1, y2);
                }

                if (minc.get(y1, y2) >= tr) {
                    cout << d << '\n';

                    continue;
                }

                if (x1 > x2) {
                    swap(x1, x2);
                }

                if (maxr.get(0, n - 1) < tr) {
                    cout << -1 << '\n';

                    continue;
                }

                int low = -1, high = max(x1, n - x2 - 1);
                while (high - low > 1) {
                    int mid = (low + high + 1) >> 1;

                    int l = max(0, x1 - mid);
                    int r = min(n - 1, x2 + mid);

                    if (maxr.get(l, r) >= tr) {
                        high = mid;
                    } else {
                        low = mid;
                    }
                }

                cout << d + high * 2 << '\n';
            }
        }
    }

    return 0;
}


/*

4 3 9
3 2 1 3 3 1
3 2 1 3 3 1
2 1
2 3
1 1
3 2 1 3 3 3
3 2 1 3 3 3
2 2
3 2 1 3 3 6

*/