#include <iostream>
#include <deque>
#include <map>
#include <cassert>
#include <queue>
#include <climits>

#define FAST_MEM (int)5e8

#ifdef FAST_MEM

char mem[FAST_MEM];
int allPos = 0;

void *operator new(size_t n) {
    char *res = mem + allPos;
    allPos += n;
    assert(allPos <= FAST_MEM);
    return (void *) res;
}

void operator delete(void *) noexcept {

}

#endif

using namespace std;

typedef pair<int, int> Cell;

int *cTI, *cTJ;

struct Node {
    int l, r, mn;
    Node *left = nullptr, *right = nullptr;

    Node(int l, int r) : l(l), r(r) {}

    void recalc() {
        mn = min(left->mn, right->mn);
    }
};

Node *build(int l, int r) {
    Node *res = new Node(l, r);
    if (l != r) {
        int m = (l + r) / 2;
        res->left = build(l, m);
        res->right = build(m + 1, r);
        res->recalc();
    }
    return res;
}

void set(Node *root, int ind, int val) {
    if (ind < root->l || ind > root->r) {
        return;
    }
    if (root->l == root->r) {
        root->mn = val;
        return;
    }
    set(root->left, ind, val);
    set(root->right, ind, val);
    root->recalc();
}

int query(Node *root, int l, int r) {
    if (r < root->l || l > root->r) {
        return INT_MAX;
    }
    if (root->l <= l && root->r >= r) {
        return root->mn;
    }
    return min(query(root->left, l, r), query(root->right, l, r));
}

int getTime(int curTime, int i, int j) {
    return curTime - max(cTI[i], cTJ[j]);
}

int dI[] = {1, -1, 0, 0};
int dJ[] = {0, 0, 1, -1};

int main() {
#ifdef LOCAL
    freopen("B.in", "r", stdin);
#endif

    ios_base::sync_with_stdio(false);
    cin.tie(0);
    cout.tie(0);

    int n, m, q;
    cin >> n >> m >> q;

    cTI = new int[n];
    cTJ = new int[m];
    fill(cTI, cTI + n, 0);
    fill(cTJ, cTJ + m, 0);
    map<Cell, int> dist;
    Node *root = build(0, n - 1);
    for (int curTime = 1; curTime <= q; ++curTime) {
        int type;
        cin >> type;
        if (type == 1) {
            int i;
            cin >> i;
            i--;
            cTI[i] = curTime;
            set(root, i, curTime);
        } else if (type == 2) {
            assert(false);
        } else if (type == 3) {
            int i1, j1, i2, j2, k;
            cin >> i1 >> j1 >> i2 >> j2 >> k;
            i1--, j1--, i2--, j2--;
            int l = min(i1, i2);
            int r = max(i1, i2);
            int mxVal = curTime - query(root, l, r);
            if (mxVal > k) {
                cout << -1 << '\n';
            } else {
                cout << abs(i2 - i1) + abs(j2 - j1) << '\n';
            }
        }
    }

    return 0;
}