#include "iostream"
#include "assert.h"

using namespace std;

const int maxn = 1e6 + 6;

struct segment_tree {
    int d[4 * maxn];
    virtual int op(int i1, int i2) = 0;
    virtual int op_extr() = 0;

    void init() {
        fill(d, d + 4 * maxn, op_extr());
    }

    void update(int v, int l, int r, int req_v, int req_color) {
        if (req_v < l || r < req_v) return;
        if (l == r) {
            d[v] = req_color;
            return;
        }
        int m = (l + r) / 2;
        update(2 * v + 1, l, m, req_v, req_color);
        update(2 * v + 2, m + 1, r, req_v, req_color);
        d[v] = op(d[2 * v + 1], d[2 * v + 2]);
    }
    void update(int req_v, int req_color) {
        update(0, 0, maxn, req_v, req_color);
    }
    int parse(int v, int l, int r, int req_l, int req_r) {
        if (req_r < l || r < req_l) return op_extr();
        if (req_l <= l && r <= req_r) {
            return d[v];
        }
        int m = (l + r) / 2;
        return op(parse(2 * v + 1, l, m, req_l, req_r),
                  parse(2 * v + 2, m + 1, r, req_l, req_r));
    }
    int parse(int req_l, int req_r) {
        return parse(0, 0, maxn, req_l, req_r);
    }
};

struct segment_tree_min: segment_tree {
    segment_tree_min() {
//        init();
//        fill(d, d + ma)
    }
    int op(int i1, int i2) {return min(i1, i2);}
    int op_extr() { return 1e9; }
};

struct segment_tree_max: segment_tree {
    segment_tree_max() {
        // init();
    }
    int op(int i1, int i2) {return max(i1, i2);}
    int op_extr() { return 0; }
};

segment_tree_min x_min;



int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(0);
    int n, m, q;
    cin >> n >> m >> q;
    int time = 0;
    for (int it = 0; it < q; it++) {
        time++;
        int req_type;
        cin >> req_type;
        if (req_type == 1) {
            int v;
            cin >> v;
            x_min.update(v, time);
        } else {
            assert(req_type == 3);
            int x1, y1, x2, y2, k;
            cin >> x1 >> y1 >> x2 >> y2 >> k; x1--; y1--; x2--; y2--;
            if (time - x_min.parse(min(x1, x2), max(x1, x2)) <= k ) {
                cout << abs(x1 - x2) + abs(y1 - y2) << "\n";
            } else {
                cout << "-1\n";
            }


        }
    }
}