#include <iostream>
#include <assert.h>
using namespace std;
const int maxn = 1e6 + 6;

struct segment_tree {
    pair<int, int> d[4 * maxn];
    virtual pair<int, int> op(pair<int, int> i1, pair<int, int> i2) = 0;
    virtual int op_extr() = 0;

  /*  void init() {
//        fill(d, d + 4 * maxn, {op_extr());
    }*/

    void update(int v, int l, int r, int req_v, int req_color) {
        if (req_v < l || r < req_v) return;
        if (l == r) {
            d[v].first = req_color;
            return;
        }
        int m = (l + r) / 2;
        update(2 * v + 1, l, m, req_v, req_color);
        update(2 * v + 2, m + 1, r, req_v, req_color);
        d[v] = op(d[2 * v + 1], d[2 * v + 2]);
    }
    void update(int req_v, int req_color) {
        update(0, 0, maxn, req_v, req_color);
    }
    pair<int, int> parse(int v, int l, int r, int req_l, int req_r) {
        if (req_r < l || r < req_l) return {op_extr(), -1};
        if (req_l <= l && r <= req_r) {
            return d[v];
        }
        int m = (l + r) / 2;
        return op(parse(2 * v + 1, l, m, req_l, req_r),
                  parse(2 * v + 2, m + 1, r, req_l, req_r));
    }
    pair<int, int> parse(int req_l, int req_r) {
        return parse(0, 0, maxn, req_l, req_r);
    }
};

struct segment_tree_min: segment_tree {
    segment_tree_min() {
//        init();
//        fill(d, d + ma)
        for (int i = 0; i < maxn; i++) {
            d[i].second = i;
        }
    }
    pair<int, int> op(pair<int, int> i1, pair<int, int> i2) {
        if (i1.first < i2.first) return i1;
        return i2;
    }
    int op_extr() { return 1e9; }
};

struct segment_tree_max: segment_tree {
    segment_tree_max() {
       // init();
        for (int i = 0; i < maxn; i++) {
            d[i].second = i;
        }
    }
    pair<int, int> op(pair<int, int> i1, pair<int, int> i2) {
        if (i1.first > i2.first) return i1;
        return i2;
    }
    int op_extr() { return 0; }
};

segment_tree_min x_min;
segment_tree_max x_max;

segment_tree_min y_min;
segment_tree_max y_max;

int n, m;

int find_max_x(int x, int y, int k, int time) {
    if (time - y_max.parse(y, y).first <= k) {
        return n - 1;
    }
    int l = x - 1;
    int r = n - 1;
    while (l != r) {
        int m = (l + r) / 2 + 1;
        if (m > r) m--;
        if (m == x - 1 || time - x_min.parse(x, m).first <= k) {
            assert(l != m);
            l = m;
        } else {
            assert(r != m - 1);
            r = m - 1;
        }
    }
    return l;
}

int find_max_y(int x, int y, int k, int time) {
    if (time - x_max.parse(x, x).first <= k) {
        return m - 1;
    }
    int l = y - 1;
    int r = m - 1;
    while (l != r) {
        int m = (l + r) / 2 + 1;
        if (m > r) m--;
        if (m == y - 1 || time - y_min.parse(y, m).first <= k) {
            assert(l != m);
            l = m;
        } else {
            assert(r != m - 1);
            r = m - 1;
        }
    }
    return l;
}

int find_min_x(int x, int y, int k, int time) {
    if (time - y_max.parse(y, y).first <= k) {
        return 0;
    }
    int l = 0;
    int r = x + 1;
    while (l != r) {
        int m = (l + r) / 2;
        if (m == x + 1 || time - x_min.parse(m, x).first <= k) {
            assert(r != m);
            r = m;
        } else {
            assert(l != m + 1);
            l = m + 1;
        }
    }
    return l;
}

int find_min_y(int x, int y, int k, int time) {
    if (time - x_max.parse(x, x).first <= k) {
        return 0;
    }
    int l = 0;
    int r = y + 1;
    while (l != r) {
        int m = (l + r) / 2;
        if (m == y + 1 || time - y_min.parse(m, y).first <= k) {
            assert(r != m);
            r = m;
        } else {
            assert(l != m + 1);
            l = m + 1;
        }
    }
    return l;
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(0);
    int q;
    cin >> n >> m >> q;
    int time = 0;
    for (int i = 0; i < q; i++) {
        time++;
        int req_type;
        cin >> req_type;
        if (req_type == 1) {
            int val;
            cin >> val; val--;
            x_min.update(val, time);
            x_max.update(val, time);
        } else if (req_type == 2) {
            int val;
            cin >> val; val--;
            y_min.update(val, time);
            y_max.update(val, time);
        } else {
            assert(req_type == 3);
            int x1, y1, x2, y2, k;
            cin >> x1 >> y1 >> x2 >> y2 >> k; x1--; x2--; y1--; y2--;
           /* cout << find_min_x(x1, y1, k, time) + 1 << " "
                    << find_max_x(x1, y1, k, time) + 1 << " "
                    << find_min_y(x1, y1, k, time) + 1 << " "
                    << find_max_y(x1, y1, k, time) + 1<< endl;*/
           int lx = max(find_min_x(x1, y1, k, time), find_min_x(x2, y2, k, time));
           int rx = min(find_max_x(x1, y1, k, time), find_max_x(x2, y2, k, time));

           int ly = max(find_min_y(x1, y1, k, time), find_min_y(x2, y2, k, time));
           int ry = min(find_max_y(x1, y1, k, time), find_max_y(x2, y2, k, time));

           if (lx <= rx) {
               auto res1 = y_min.parse(y1, y2);
               if (time - res1.first <= k) {
                   cout << abs(x2 - x1) + abs(y2 - y1) << "\n";
                   continue;
               }
               /*auto res2 = x_max.parse(lx, rx);
               if (time - res2.first <= k) {
                   cout << abs(x1 - res2.second) + abs(y2 - y1) + abs(x2 - res2.second) << "\n";
                   continue;
               }*/
               int res = 1e9;
               for (int x = lx; x <= rx; x++) {
                   if (time - x_min.parse(x, x).first <= k) {
                       res = min(res, abs(x1 - x) + abs(y1 - y2) + abs(x2 - x));
                   }
               }
               if (res != 1e9) {
                   cout << res << "\n";
                   continue;
               }
           }
            if (ly <= ry) {
                auto res1 = x_min.parse(x1, x2);
                if (time - res1.first <= k) {
                    cout << abs(y2 - y1) + abs(x2 - x1) << "\n";
                    continue;
                }
                /*auto res2 = y_max.parse(ly, ry);
                if (time - res2.first <= k) {
                    cout << abs(y1 - res2.second) + abs(x2 - x1) + abs(y2 - res2.second) << "\n";
                    continue;
                }*/
                int res = 1e9;
                for (int y = ly; y <= ry; y++) {
                    if (time - y_min.parse(y, y).first <= k) {
                        res = min(res, abs(y1 - y) + abs(x1 - x2) + abs(y2 - y));
                    }
                }
                if (res != 1e9) {
                    cout << res << "\n";
                    continue;
                }
            }
           cout << -1 << "\n";

           /*if (lx <= rx && (time - y_max.parse(lx, rx).first <= k || time - x_min.parse(lx, rx).first <= k)) {

               cout << abs(x2 - x1) + abs(y2 - y1) << "\n";
           } else if (ly <= ry && (time - x_max.parse(ly, ry).first <= k || time - y_min.parse(ly, ry).first <= k)) {
               cout << abs(x2 - x1) + abs(y2 - y1) << "\n";
           } else {
               cout << -1 << "\n";
           }*/
        }
    }
}